﻿
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Microsoft.Win32;
using SlnMesnac.LabelPrint.BarTender;
using SlnMesnac.LabelPrint.Common;
using SlnMesnac.LabelPrint.FileOperate;
using SlnMesnac.LabelPrint.Log4net;
using SlnMesnac.LabelPrint.Model;
using SlnMesnac.LabelPrint.TaskQueue;
using SlnMesnac.LabelPrint.Timer;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading;

namespace SlnMesnac.LabelPrint.viewModel
{
    public class MainWindowViewModel: ViewModelBase
    {
        private readonly LogHelper logHelper = LogHelper.Instance;
        private readonly FileHelper fileHelper = FileHelper.Instance;
        private readonly TaskHelper taskHelper = TaskHelper.Instance;
        private readonly JsonChange jsonChange = JsonChange.Instance;
        private readonly PrintManager printManager = PrintManager.Instance;
        private readonly PrintTimer printTimer = PrintTimer.Instance;
        private ObservableCollection<dynamic> listItems = new ObservableCollection<dynamic>(); 
        private ObservableCollection<BaseLabelInfo> labelItems = new ObservableCollection<BaseLabelInfo>();
        #region 参数定义
        /// <summary>
        /// 日志打印ListBox
        /// </summary>
        private IEnumerable logInfoListBox;
        public IEnumerable LogInfoListBox
        {
            get { return logInfoListBox; }
            set { logInfoListBox = value; RaisePropertyChanged(() => LogInfoListBox); }
        }

        /// <summary>
        /// 标签信息DataGrid
        /// </summary>
        private IEnumerable labelInfoDataGrid;

        public IEnumerable LabelInfoDataGrid
        {
            get { return labelInfoDataGrid; }
            set { labelInfoDataGrid = value; RaisePropertyChanged(() => LabelInfoDataGrid); }

        }
        #endregion

        #region 事件定义
        /// <summary>
        /// 导入文件
        /// </summary>
        public RelayCommand ImportFilesCommand { get; set; }

        /// <summary>
        /// 手动执行
        /// </summary>
        public RelayCommand ManualExecutionCommand { get; set; }

        /// <summary>
        /// 开始打印
        /// </summary>
        public RelayCommand StartPrintCommand { get; set; }

        /// <summary>
        /// 停止打印
        /// </summary>
        public RelayCommand StopPrintCommand { get; set; }
        #endregion

        public MainWindowViewModel()
        {
            logHelper.Info("初始化成功！！！");
            try
            {
                taskHelper.ClearLabelInfoDataGridEvent += ClearLabelInfoDataGrid;
                printTimer.LogRefreshEvent += PrintMessageToListBox;
                ImportFilesCommand = new RelayCommand(ImportFile);
                ManualExecutionCommand = new RelayCommand(ManualExecution);
                StartPrintCommand = new RelayCommand(StartPrint);
                StopPrintCommand = new RelayCommand(StopPrint);
            }catch(Exception ex)
            {
                logHelper.Error("初始化事件绑定异常", ex);
            }
        }

        /// <summary>
        /// 导入文件事件实现
        /// </summary>
        private void ImportFile()
        {

            try
            {
                string fileName = "";
                OpenFileDialog openFileDialog = new OpenFileDialog();
                openFileDialog.Multiselect = true;      //该值确定是否可以选择多个文件
                openFileDialog.Title = "请选择文件";     //弹窗的标题
                openFileDialog.InitialDirectory = "C:\\Users\\Administrator\\Desktop";       //默认打开的文件夹的位置
                openFileDialog.Filter = "MicroSoft Excel文件(*.xlsx)|*.xlsx|所有文件(*.*)|*.*";       //筛选文件

                if (openFileDialog.ShowDialog() == true)
                {
                    fileName = openFileDialog.FileName;

                    PrintMessageToListBox($"文件上传成功，路径：{fileName}");
                    
                    //读取Excel内容
                    List<BaseLabelInfo> labelInfos = fileHelper.ReadExceFile(fileName);
                    RefreshLabelInfoDataGrid(labelInfos);
                    Thread.Sleep(1000);
                    PrintMessageToListBox($"文件解析完成：{jsonChange.ModeToJson(labelInfos)}");
                    taskHelper.AddTask(labelInfos);
                }


            }catch(Exception ex)
            {
                this.PrintMessageToListBox($"导入文件事件实现异常：{ex.Message}");
            }
        }

        /// <summary>
        /// 手动打印
        /// </summary>
        private void ManualExecution()
        {
            var info = taskHelper.GetTask();
            if (info == null)
            {
                this.PrintMessageToListBox("未获取到需要打印的任务");
            }
            else
            {
                this.PrintMessageToListBox($"开始打印标签信息：{jsonChange.ModeToJson(info)}");
                Thread.Sleep(500);
                if (info.column_A == null)
                {
                    this.PrintMessageToListBox($"{info.productType}打印任务执行失败，A列数据为空");
                    return;
                }
                var printResult = printManager.Print(info);
                if (printResult)
                {
                    this.PrintMessageToListBox($"{info.productType}打印{info.column_A.ToString()}；打印成功");
                }
                else
                {
                    this.PrintMessageToListBox($"{info.column_A.ToString()}；打印失败");
                }

                taskHelper.RemoveTask(info);
            }
        }

        /// <summary>
        /// 开始打印事件实现
        /// </summary>
        private void StartPrint()
        {
            printTimer.StartPrint();
        }

        /// <summary>
        /// 停止打印事件实现
        /// </summary>
        private void StopPrint()
        {
            printTimer.StopPrint();
        }

        /// <summary>
        /// 加载标签信息DataGrid
        /// </summary>
        /// <param name="labelInfos"></param>
        private void RefreshLabelInfoDataGrid(List<BaseLabelInfo> labelInfos)
        {
            if (labelInfos == null) return;
            App.Current.Dispatcher.BeginInvoke((Action)(() =>
            {
                //labelItems = new ObservableCollection<BaseLabelInfo>();
                foreach (var item in labelInfos)
                {
                    labelItems.Add(item);
                }

                LabelInfoDataGrid = labelItems;
            }));
        }

        /// <summary>
        /// 打印完成后回调该函数
        /// </summary>
        /// <param name="labelInfo"></param>
        private void ClearLabelInfoDataGrid(BaseLabelInfo labelInfo)
        {
            if (labelInfo == null) return;
            App.Current.Dispatcher.BeginInvoke((Action)(() =>
            {
                labelItems.Remove(labelInfo);

                LabelInfoDataGrid = labelItems;
            }));
        }

        /// <summary>
        /// listBox绑定日志
        /// </summary>
        /// <param name="message"></param>
        private void PrintMessageToListBox(string message)
        {

            lock (string.Empty)
            {
                try
                {
                    listItems.Add($"{DateTime.Now.ToString("HH:mm:ss")}==>{message}");

                    LogInfoListBox = listItems.OrderByDescending(x => x);
                }
                catch (Exception ex)
                {
                    logHelper.Error("日志数据绑定异常", ex);
                }
            }
        }
    }
}
